'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _memwatchNext = require('memwatch-next');

var _memwatchNext2 = _interopRequireDefault(_memwatchNext);

var _IterationBlock = require('./IterationBlock');

var _IterationBlock2 = _interopRequireDefault(_IterationBlock);

var _getSubsequentHeapGrowths = require('./getSubsequentHeapGrowths');

var _getSubsequentHeapGrowths2 = _interopRequireDefault(_getSubsequentHeapGrowths);

var _leakErrorFactory = require('./leakErrorFactory');

var _leakErrorFactory2 = _interopRequireDefault(_leakErrorFactory);

var _saveHeapDiffs = require('./saveHeapDiffs');

var _saveHeapDiffs2 = _interopRequireDefault(_saveHeapDiffs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var LeakageRunner = function () {
  function LeakageRunner(iterationCount, iteratorFunc, onDoneFunc) {
    _classCallCheck(this, LeakageRunner);

    this.garbageCollections = 6;
    this.throwOnSubsequentHeapGrows = 4;
    this.heapDiffs = [];

    this.iteratorFunc = iteratorFunc;
    this.iterationCount = iterationCount > this.garbageCollections ? iterationCount : this.garbageCollections;

    this.iterationBlocks = createIterationBlocks(this.iterationCount, this.garbageCollections, iteratorFunc);
    this.iterationBlockDone = this.iterationBlockDone.bind(this);
    this.iterationBlocksDone = 0;
    this.iteratorIsAsync = false;
    this.onDoneFunc = onDoneFunc;
  }

  _createClass(LeakageRunner, [{
    key: 'destruct',
    value: function destruct() {
      // Make sure we clean up our stuff

      this.heapDiffs = null;
      this.iteratorFunc = null;
      this.onDoneFunc = null;

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = this.iterationBlocks[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var iterationBlock = _step.value;

          iterationBlock.destruct();
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      _memwatchNext2.default.gc();
    }
  }, {
    key: 'iterate',
    value: function iterate() {
      this.error = null;
      _memwatchNext2.default.gc();
      this.startNextIterationBlock();
    }
  }, {
    key: 'startNextIterationBlock',
    value: function startNextIterationBlock() {
      if (this.iterationBlocksDone >= this.iterationBlocks.length) {
        return this.allIterationBlocksDone();
      }

      this.currentHeapDiff = new _memwatchNext2.default.HeapDiff();

      // this.iterationBlockDone has already been bound
      this.iterationBlocks[this.iterationBlocksDone++].iterate(this.iterationBlockDone);
    }
  }, {
    key: 'iterationBlockDone',
    value: function iterationBlockDone(error, iteratorIsAsync) {
      // Seems to work better with multiple GCs. Maybe `memwatch.gc()` only triggers a partial GC...
      _memwatchNext2.default.gc();
      _memwatchNext2.default.gc();
      _memwatchNext2.default.gc();
      this.heapDiffs.push(this.currentHeapDiff.end());
      this.iteratorIsAsync = iteratorIsAsync;

      if (error) {
        this.error = error;
        this.allIterationBlocksDone();
      } else {
        this.startNextIterationBlock();
      }
    }
  }, {
    key: 'allIterationBlocksDone',
    value: function allIterationBlocksDone() {
      if (!this.error) {
        this.allIterationBlocksSuccess();
      }
      this.onDoneFunc(this.error);
    }
  }, {
    key: 'allIterationBlocksSuccess',
    value: function allIterationBlocksSuccess() {
      (0, _saveHeapDiffs2.default)(this.heapDiffs, this.throwOnSubsequentHeapGrows);

      var subsequentHeapGrowths = (0, _getSubsequentHeapGrowths2.default)(this.heapDiffs);

      if (subsequentHeapGrowths.length >= this.throwOnSubsequentHeapGrows) {
        var newHeapError = (0, _leakErrorFactory2.default)(this.iterationCount, this.iterationBlocks.length);
        this.error = newHeapError(subsequentHeapGrowths);
      }
    }
  }]);

  return LeakageRunner;
}();

/**
 * Split complete iteration into multiple chunks.
 */


exports.default = LeakageRunner;
function createIterationBlocks(totalIterations, blocksToCreate, iteratorFunc) {
  var blocks = [];

  for (var index = 0; index < blocksToCreate; index++) {
    var iterations = Math.round((index + 1) / blocksToCreate * totalIterations) - Math.round(index / blocksToCreate * totalIterations);
    blocks.push(new _IterationBlock2.default(iterations, iteratorFunc));
  }

  return blocks;
}