'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = iterate;

var _LeakageRunner = require('./LeakageRunner');

var _LeakageRunner2 = _interopRequireDefault(_LeakageRunner);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var runningLeakTests = 0;

/**
 * @param {number} iterationCount
 * @param {Function} iteratorFunc
 */
function iterate(iterationCount, iteratorFunc) {
  var error = null;
  var onDoneHasRun = false;
  var onAsyncCompletion = function onAsyncCompletion() {};

  var runner = new _LeakageRunner2.default(iterationCount, iteratorFunc, onDone);

  function onDone(iterationError) {
    error = iterationError;
    runningLeakTests--;
    onDoneHasRun = true;
    runner.destruct();
    onAsyncCompletion(iterationError);
  }

  if (runningLeakTests > 0) {
    throw new Error(`Multiple concurrent leak tests running. This will ruin the heap diffs. Make sure the tests are run strictly sequential.`);
  }

  runningLeakTests++;
  runner.iterate();

  if (onDoneHasRun) {
    // Iteration performed synchronously
    if (error) {
      throw error;
    }
  } else {
    return new Promise(function (resolve, reject) {
      onAsyncCompletion = function onAsyncCompletion(error) {
        if (error) {
          reject(error);
        } else {
          resolve();
        }
      };
    });
  }
}